/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.
 */

#ifndef CONFIGURATION_OPTIONS_H
#define CONFIGURATION_OPTIONS_H

#include <string>

#include <boost/program_options.hpp>
#include <boost/utility.hpp>

#include "config/configuration.h"
#include "config/option/configurationoption.h"
#include "config/option/hostconfigurationoption.h"

//-----------------------------------------------------------------------------
// ConfigurationOptions
//-----------------------------------------------------------------------------

/**
 * @brief Options for configuration.
 */
class ConfigurationOptions : boost::noncopyable
{
public:
    ConfigurationOptions();
    virtual ~ConfigurationOptions();

    boost::program_options::options_description get_generic_options() const;
    boost::program_options::options_description get_configuration_options() const;

    bool parse_generic_options(
            const boost::program_options::variables_map &vm,
            Configuration *configuration
    );
    bool parse_configuration_options(
            const boost::program_options::variables_map &vm,
            Configuration *configuration
    );

    bool halt_on_generic_options(
            const boost::program_options::variables_map &vm
    ) const;

private:
    bool parse_hosts_options(
            const boost::program_options::variables_map &vm,
            Configuration *configuration
    );

private:
    ConfigurationOptionList GenericOptions;
    ConfigurationOptionList ConfigOptions;
    HostConfigurationOptionList HostOptions;

    const std::string HelpCmdStr;
    const std::string HelpCmdDesc;

};

#endif // CONFIGURATION_OPTIONS_H
